@@ -12,11 +12,11 @@ mod errors;
 mod register_op_store;
 
 use data_store::to_db_key::ToDbKey;
+pub use data_store::used_space::UsedSpace;
 pub(crate) use data_store::{
     data::{Data, DataId},
-    used_space::UsedSpace,
     DataStore, Subdir,
 };
-pub(crate) use errors::Error;
-use errors::Result;
+pub(crate) use errors::Result;
+pub(crate) use errors::{convert_to_error_message, Error};
 pub(crate) use register_op_store::RegisterOpStore;
