@@ -6,12 +6,12 @@
 // KIND, either express or implied. Please review the Licences for the specific language governing
 // permissions and limitations relating to use of the SAFE Network Software.
 
-use super::{chunk_records::ChunkRecords, register_storage::RegisterStorage};
+use super::chunk_records::ChunkRecords;
 use crate::messaging::{
     data::{DataCmd, DataExchange, DataQuery},
     Authority, DataSigned, EndUser, MessageId,
 };
-use crate::node::{node_ops::NodeDuty, Error, Result};
+use crate::node::{network::Network, node_ops::NodeDuty, Error, Result};
 use crate::routing::Prefix;
 use crate::types::PublicKey;
 use tracing::info;
@@ -20,14 +20,16 @@ use tracing::info;
 /// that are only managed at Elders.
 pub(super) struct ElderStores {
     chunk_records: ChunkRecords,
-    register_storage: RegisterStorage,
+    // TODO: this is needed to access RegsiterStorage for DataExchange flows
+    // This should be removed once chunks are all in routing.
+    network: Network,
 }
 
 impl ElderStores {
-    pub(super) fn new(chunk_records: ChunkRecords, register_storage: RegisterStorage) -> Self {
+    pub(super) fn new(chunk_records: ChunkRecords, network: Network) -> Self {
         Self {
             chunk_records,
-            register_storage,
+            network,
         }
     }
 
@@ -35,15 +37,14 @@ impl ElderStores {
         &self,
         query: DataQuery,
         msg_id: MessageId,
-        requester: PublicKey,
+        _requester_pk: PublicKey,
         origin: EndUser,
     ) -> Result<NodeDuty> {
         match &query {
             DataQuery::Blob(read) => self.chunk_records.read(read, msg_id, origin).await,
-            DataQuery::Register(read) => {
-                self.register_storage
-                    .read(read, msg_id, requester, origin)
-                    .await
+            DataQuery::Register(_read) => {
+                // This has been moved to routing/core/msg_handling
+                unimplemented!("Not reading Register in node duty flow anymore")
             }
         }
     }
@@ -63,11 +64,9 @@ impl ElderStores {
                     .write(write, msg_id, data_auth, origin)
                     .await
             }
-            DataCmd::Register(write) => {
-                info!("Writing Register");
-                self.register_storage
-                    .write(msg_id, origin, write, data_auth)
-                    .await
+            DataCmd::Register(_write) => {
+                // This has been moved to routing/core/msg_handling
+                unimplemented!("Not writing Register in node duty flow anymore");
             }
         }
     }
@@ -76,11 +75,12 @@ impl ElderStores {
         &self.chunk_records
     }
 
-    // NB: Not yet including Register metadata.
     pub(super) async fn get_data_of(&self, prefix: Prefix) -> Result<DataExchange> {
         // Prepare chunk_records, map and sequence data
         let chunk_data = self.chunk_records.get_data_of(prefix).await;
-        let reg_data = self.register_storage.get_data_of(prefix).await?;
+
+        let register_storage = self.network.get_register_storage().await;
+        let reg_data = register_storage.get_data_of(prefix).await?;
 
         Ok(DataExchange {
             chunk_data,
@@ -88,9 +88,13 @@ impl ElderStores {
         })
     }
 
+    // TODO: This should be moved into routing
     pub(super) async fn update(&mut self, data: DataExchange) -> Result<(), Error> {
         // todo: all this can be done in parallel
-        self.register_storage.update(data.reg_data).await?;
+
+        let register_storage = self.network.get_register_storage().await;
+
+        register_storage.update(data.reg_data)?;
         self.chunk_records.update(data.chunk_data).await;
         Ok(())
     }
