@@ -9,6 +9,7 @@
 #![allow(dead_code, unused_imports)]
 
 use super::{Comm, Command, Core, Dispatcher};
+use crate::dbs::UsedSpace;
 use crate::messaging::{
     node::{
         JoinAsRelocatedRequest, JoinRequest, JoinResponse, KeyedSig, MembershipState, Network,
@@ -19,8 +20,11 @@ use crate::messaging::{
     Authority, DstLocation, MessageId, MessageType, MsgKind, NodeSigned, SectionAuthorityProvider,
     SectionSigned as MsgKindSectionSigned, WireMsg,
 };
+use crate::node::RegisterStorage;
+
 use crate::routing::{
     core::{ConnectionEvent, RESOURCE_PROOF_DATA_SIZE, RESOURCE_PROOF_DIFFICULTY},
+    create_test_used_space_and_root_storage,
     dkg::{
         test_utils::{prove, section_signed},
         ProposalUtils,
@@ -39,18 +43,21 @@ use crate::routing::{
     MIN_ADULT_AGE, MIN_AGE,
 };
 use crate::types::{Keypair, PublicKey};
-use anyhow::{anyhow, Result};
+use anyhow::{anyhow, Context, Result};
 use assert_matches::assert_matches;
 use ed25519_dalek::Signer;
 use rand::rngs::OsRng;
+use rand::{distributions::Alphanumeric, Rng};
 use resource_proof::ResourceProof;
 use secured_linked_list::SecuredLinkedList;
+use std::path::Path;
 use std::{
     collections::{BTreeSet, HashSet},
     iter,
     net::Ipv4Addr,
     ops::Deref,
 };
+use tempfile::tempdir;
 use tokio::{
     sync::mpsc,
     time::{timeout, Duration},
@@ -59,14 +66,17 @@ use xor_name::{Prefix, XorName};
 
 static TEST_EVENT_CHANNEL_SIZE: usize = 20;
 
-#[tokio::test]
+#[tokio::test(flavor = "multi_thread")]
 async fn receive_matching_get_section_request_as_elder() -> Result<()> {
     let node = create_node(MIN_ADULT_AGE, None);
     let node_name = node.name();
+    let (used_space, root_storage_dir) = create_test_used_space_and_root_storage()?;
     let core = Core::first_node(
         create_comm().await?,
         node,
         mpsc::channel(TEST_EVENT_CHANNEL_SIZE).0,
+        used_space,
+        root_storage_dir,
     )?;
     let node_section_pk = *core.section().chain().last_key();
     let section_auth = core.section().authority_provider().clone();
@@ -114,7 +124,7 @@ async fn receive_matching_get_section_request_as_elder() -> Result<()> {
     Ok(())
 }
 
-#[tokio::test]
+#[tokio::test(flavor = "multi_thread")]
 async fn receive_mismatching_get_section_request_as_adult() -> Result<()> {
     let good_prefix = Prefix::default().pushed(false);
 
@@ -124,13 +134,16 @@ async fn receive_mismatching_get_section_request_as_adult() -> Result<()> {
 
     let node = create_node(MIN_ADULT_AGE, Some(good_prefix));
     let node_name = node.name();
+    let (used_space, root_storage_dir) = create_test_used_space_and_root_storage()?;
     let core = Core::new(
         create_comm().await?,
         node,
         section,
         None,
         mpsc::channel(TEST_EVENT_CHANNEL_SIZE).0,
-    );
+        used_space,
+        root_storage_dir,
+    )?;
     let node_section_pk = *core.section().chain().last_key();
     let dispatcher = Dispatcher::new(core);
 
@@ -186,13 +199,16 @@ async fn receive_mismatching_get_section_request_as_adult() -> Result<()> {
 // `Redirect` response containing addresses of nodes in a section that is closer to the joining
 // name.
 
-#[tokio::test]
+#[tokio::test(flavor = "multi_thread")]
 async fn receive_join_request_without_resource_proof_response() -> Result<()> {
     let node = create_node(FIRST_SECTION_MIN_AGE, None);
+    let (used_space, root_storage_dir) = create_test_used_space_and_root_storage()?;
     let core = Core::first_node(
         create_comm().await?,
         node,
         mpsc::channel(TEST_EVENT_CHANNEL_SIZE).0,
+        used_space,
+        root_storage_dir,
     )?;
     let dispatcher = Dispatcher::new(core);
 
@@ -239,13 +255,16 @@ async fn receive_join_request_without_resource_proof_response() -> Result<()> {
     Ok(())
 }
 
-#[tokio::test]
+#[tokio::test(flavor = "multi_thread")]
 async fn receive_join_request_with_resource_proof_response() -> Result<()> {
     let node = create_node(FIRST_SECTION_MIN_AGE, None);
+    let (used_space, root_storage_dir) = create_test_used_space_and_root_storage()?;
     let core = Core::first_node(
         create_comm().await?,
         node,
         mpsc::channel(TEST_EVENT_CHANNEL_SIZE).0,
+        used_space,
+        root_storage_dir,
     )?;
     let dispatcher = Dispatcher::new(core);
 
@@ -310,7 +329,7 @@ async fn receive_join_request_with_resource_proof_response() -> Result<()> {
     Ok(())
 }
 
-#[tokio::test]
+#[tokio::test(flavor = "multi_thread")]
 async fn receive_join_request_from_relocated_node() -> Result<()> {
     let (section_auth, mut nodes) = create_section_auth();
 
@@ -321,13 +340,16 @@ async fn receive_join_request_from_relocated_node() -> Result<()> {
     let (section, section_key_share) = create_section(&sk_set, &section_auth)?;
     let node = nodes.remove(0);
     let node_name = node.name();
+    let (used_space, root_storage_dir) = create_test_used_space_and_root_storage()?;
     let core = Core::new(
         create_comm().await?,
         node,
         section,
         Some(section_key_share),
         mpsc::channel(TEST_EVENT_CHANNEL_SIZE).0,
-    );
+        used_space,
+        root_storage_dir,
+    )?;
     let dispatcher = Dispatcher::new(core);
 
     let relocated_node_old_keypair =
@@ -404,19 +426,22 @@ async fn receive_join_request_from_relocated_node() -> Result<()> {
     Ok(())
 }
 
-#[tokio::test]
+#[tokio::test(flavor = "multi_thread")]
 async fn aggregate_proposals() -> Result<()> {
     let (section_auth, nodes) = create_section_auth();
     let sk_set = SecretKeySet::random();
     let pk_set = sk_set.public_keys();
     let (section, section_key_share) = create_section(&sk_set, &section_auth)?;
+    let (used_space, root_storage_dir) = create_test_used_space_and_root_storage()?;
     let core = Core::new(
         create_comm().await?,
         nodes[0].clone(),
         section.clone(),
         Some(section_key_share),
         mpsc::channel(TEST_EVENT_CHANNEL_SIZE).0,
-    );
+        used_space,
+        root_storage_dir,
+    )?;
     let dispatcher = Dispatcher::new(core);
 
     let new_peer = create_peer(MIN_AGE);
@@ -429,7 +454,7 @@ async fn aggregate_proposals() -> Result<()> {
     for (index, node) in nodes.iter().enumerate().take(THRESHOLD) {
         let sig_share = proposal.prove(pk_set.clone(), index, &sk_set.secret_key_share(index))?;
         let wire_msg = WireMsg::single_src(
-            &node,
+            node,
             DstLocation::DirectAndUnrouted(*section.chain().last_key()),
             NodeMsg::Propose {
                 content: proposal.clone(),
@@ -479,7 +504,7 @@ async fn aggregate_proposals() -> Result<()> {
     Ok(())
 }
 
-#[tokio::test]
+#[tokio::test(flavor = "multi_thread")]
 async fn handle_agreement_on_online() -> Result<()> {
     let (event_tx, mut event_rx) = mpsc::channel(TEST_EVENT_CHANNEL_SIZE);
 
@@ -489,13 +514,16 @@ async fn handle_agreement_on_online() -> Result<()> {
     let sk_set = SecretKeySet::random();
     let (section, section_key_share) = create_section(&sk_set, &section_auth)?;
     let node = nodes.remove(0);
+    let (used_space, root_storage_dir) = create_test_used_space_and_root_storage()?;
     let core = Core::new(
         create_comm().await?,
         node,
         section,
         Some(section_key_share),
         event_tx,
-    );
+        used_space,
+        root_storage_dir,
+    )?;
     let dispatcher = Dispatcher::new(core);
 
     let new_peer = create_peer(MIN_AGE);
@@ -511,7 +539,7 @@ async fn handle_agreement_on_online() -> Result<()> {
     Ok(())
 }
 
-#[tokio::test]
+#[tokio::test(flavor = "multi_thread")]
 async fn handle_agreement_on_online_of_elder_candidate() -> Result<()> {
     let sk_set = SecretKeySet::random();
     let chain = SecuredLinkedList::new(sk_set.secret_key().public_key());
@@ -546,13 +574,16 @@ async fn handle_agreement_on_online_of_elder_candidate() -> Result<()> {
     let node = nodes.remove(0);
     let node_name = node.name();
     let section_key_share = create_section_key_share(&sk_set, 0);
+    let (used_space, root_storage_dir) = create_test_used_space_and_root_storage()?;
     let core = Core::new(
         create_comm().await?,
         node,
         section,
         Some(section_key_share),
         mpsc::channel(TEST_EVENT_CHANNEL_SIZE).0,
-    );
+        used_space,
+        root_storage_dir,
+    )?;
     let dispatcher = Dispatcher::new(core);
 
     // Handle agreement on Online of a peer that is older than the youngest
@@ -708,13 +739,16 @@ async fn handle_agreement_on_online_of_rejoined_node(phase: NetworkPhase, age: u
     // Make a Node
     let (event_tx, _event_rx) = mpsc::channel(TEST_EVENT_CHANNEL_SIZE);
     let node = nodes.remove(0);
+    let (used_space, root_storage_dir) = create_test_used_space_and_root_storage()?;
     let state = Core::new(
         create_comm().await?,
         node,
         section,
         Some(section_key_share),
         event_tx,
-    );
+        used_space,
+        root_storage_dir,
+    )?;
     let dispatcher = Dispatcher::new(state);
 
     // Simulate peer with the same name is rejoin and verify resulted behaviours.
@@ -736,27 +770,27 @@ async fn handle_agreement_on_online_of_rejoined_node(phase: NetworkPhase, age: u
     Ok(())
 }
 
-#[tokio::test]
+#[tokio::test(flavor = "multi_thread")]
 async fn handle_agreement_on_online_of_rejoined_node_with_high_age_in_startup() -> Result<()> {
     handle_agreement_on_online_of_rejoined_node(NetworkPhase::Startup, 16).await
 }
 
-#[tokio::test]
+#[tokio::test(flavor = "multi_thread")]
 async fn handle_agreement_on_online_of_rejoined_node_with_high_age_after_startup() -> Result<()> {
     handle_agreement_on_online_of_rejoined_node(NetworkPhase::Regular, 16).await
 }
 
-#[tokio::test]
+#[tokio::test(flavor = "multi_thread")]
 async fn handle_agreement_on_online_of_rejoined_node_with_low_age_in_startup() -> Result<()> {
     handle_agreement_on_online_of_rejoined_node(NetworkPhase::Startup, 8).await
 }
 
-#[tokio::test]
+#[tokio::test(flavor = "multi_thread")]
 async fn handle_agreement_on_online_of_rejoined_node_with_low_age_after_startup() -> Result<()> {
     handle_agreement_on_online_of_rejoined_node(NetworkPhase::Regular, 8).await
 }
 
-#[tokio::test]
+#[tokio::test(flavor = "multi_thread")]
 async fn handle_agreement_on_offline_of_non_elder() -> Result<()> {
     let (section_auth, mut nodes) = create_section_auth();
     let sk_set = SecretKeySet::random();
@@ -770,13 +804,16 @@ async fn handle_agreement_on_offline_of_non_elder() -> Result<()> {
 
     let (event_tx, mut event_rx) = mpsc::channel(TEST_EVENT_CHANNEL_SIZE);
     let node = nodes.remove(0);
+    let (used_space, root_storage_dir) = create_test_used_space_and_root_storage()?;
     let core = Core::new(
         create_comm().await?,
         node,
         section,
         Some(section_key_share),
         event_tx,
-    );
+        used_space,
+        root_storage_dir,
+    )?;
     let dispatcher = Dispatcher::new(core);
 
     let node_state = NodeState {
@@ -799,7 +836,7 @@ async fn handle_agreement_on_offline_of_non_elder() -> Result<()> {
     Ok(())
 }
 
-#[tokio::test]
+#[tokio::test(flavor = "multi_thread")]
 async fn handle_agreement_on_offline_of_elder() -> Result<()> {
     let (section_auth, mut nodes) = create_section_auth();
     let sk_set = SecretKeySet::random();
@@ -822,6 +859,7 @@ async fn handle_agreement_on_offline_of_elder() -> Result<()> {
 
     // Create our node
     let (event_tx, mut event_rx) = mpsc::channel(TEST_EVENT_CHANNEL_SIZE);
+    let (used_space, root_storage_dir) = create_test_used_space_and_root_storage()?;
     let node = nodes.remove(0);
     let node_name = node.name();
     let core = Core::new(
@@ -830,7 +868,9 @@ async fn handle_agreement_on_offline_of_elder() -> Result<()> {
         section,
         Some(section_key_share),
         event_tx,
-    );
+        used_space,
+        root_storage_dir,
+    )?;
     let dispatcher = Dispatcher::new(core);
 
     // Handle agreement on the Offline proposal
@@ -899,12 +939,12 @@ async fn handle_agreement_on_offline_of_elder() -> Result<()> {
     Ok(())
 }
 
-#[tokio::test]
+#[tokio::test(flavor = "multi_thread")]
 async fn handle_untrusted_message_from_peer() -> Result<()> {
     handle_untrusted_message(UntrustedMessageSource::Peer).await
 }
 
-#[tokio::test]
+#[tokio::test(flavor = "multi_thread")]
 async fn handle_untrusted_accumulated_message() -> Result<()> {
     handle_untrusted_message(UntrustedMessageSource::Accumulation).await
 }
@@ -930,13 +970,16 @@ async fn handle_untrusted_message(source: UntrustedMessageSource) -> Result<()>
 
     let node = create_node(MIN_ADULT_AGE, None);
     let node_name = node.name();
+    let (used_space, root_storage_dir) = create_test_used_space_and_root_storage()?;
     let core = Core::new(
         create_comm().await?,
         node,
         section.clone(),
         None,
         mpsc::channel(TEST_EVENT_CHANNEL_SIZE).0,
-    );
+        used_space,
+        root_storage_dir,
+    )?;
     let dispatcher = Dispatcher::new(core);
 
     // Create a any message signed by a key not known to the node.
@@ -1023,7 +1066,7 @@ async fn handle_untrusted_message(source: UntrustedMessageSource) -> Result<()>
     Ok(())
 }
 
-#[tokio::test]
+#[tokio::test(flavor = "multi_thread")]
 async fn handle_bounced_untrusted_message() -> Result<()> {
     let (section_auth, mut nodes, sk_set0) =
         gen_section_authority_provider(Prefix::default(), ELDER_SIZE);
@@ -1055,13 +1098,16 @@ async fn handle_bounced_untrusted_message() -> Result<()> {
     };
 
     // Create our node.
+    let (used_space, root_storage_dir) = create_test_used_space_and_root_storage()?;
     let core = Core::new(
         create_comm().await?,
         node,
         section.clone(),
         Some(section_key_share),
         mpsc::channel(TEST_EVENT_CHANNEL_SIZE).0,
-    );
+        used_space,
+        root_storage_dir,
+    )?;
     let dispatcher = Dispatcher::new(core);
 
     // Create the bounced message, indicating the last key the peer knows is `pk0`
@@ -1113,7 +1159,7 @@ async fn handle_bounced_untrusted_message() -> Result<()> {
     Ok(())
 }
 
-#[tokio::test]
+#[tokio::test(flavor = "multi_thread")]
 async fn handle_sync() -> Result<()> {
     // Create first `Section` with a chain of length 2
     let sk0 = bls::SecretKey::random();
@@ -1134,13 +1180,16 @@ async fn handle_sync() -> Result<()> {
     let (event_tx, mut event_rx) = mpsc::channel(TEST_EVENT_CHANNEL_SIZE);
     let section_key_share = create_section_key_share(&sk1_set, 0);
     let node = nodes.remove(0);
+    let (used_space, root_storage_dir) = create_test_used_space_and_root_storage()?;
     let core = Core::new(
         create_comm().await?,
         node,
         old_section,
         Some(section_key_share),
         event_tx,
-    );
+        used_space,
+        root_storage_dir,
+    )?;
     let dispatcher = Dispatcher::new(core);
 
     // Create new `Section` as a successor to the previous one.
@@ -1199,7 +1248,7 @@ async fn handle_sync() -> Result<()> {
     Ok(())
 }
 
-#[tokio::test]
+#[tokio::test(flavor = "multi_thread")]
 async fn handle_untrusted_sync() -> Result<()> {
     let sk0 = bls::SecretKey::random();
     let pk0 = sk0.public_key();
@@ -1230,7 +1279,16 @@ async fn handle_untrusted_sync() -> Result<()> {
 
     let (event_tx, mut event_rx) = mpsc::channel(TEST_EVENT_CHANNEL_SIZE);
     let node = create_node(MIN_ADULT_AGE, None);
-    let core = Core::new(create_comm().await?, node, old_section, None, event_tx);
+    let (used_space, root_storage_dir) = create_test_used_space_and_root_storage()?;
+    let core = Core::new(
+        create_comm().await?,
+        node,
+        old_section,
+        None,
+        event_tx,
+        used_space,
+        root_storage_dir,
+    )?;
     let dispatcher = Dispatcher::new(core);
 
     let sender = create_node(MIN_ADULT_AGE, None);
@@ -1285,7 +1343,7 @@ async fn handle_untrusted_sync() -> Result<()> {
     Ok(())
 }
 
-#[tokio::test]
+#[tokio::test(flavor = "multi_thread")]
 async fn handle_bounced_untrusted_sync() -> Result<()> {
     let sk0 = bls::SecretKey::random();
     let pk0 = sk0.public_key();
@@ -1310,13 +1368,16 @@ async fn handle_bounced_untrusted_sync() -> Result<()> {
     let (event_tx, _) = mpsc::channel(TEST_EVENT_CHANNEL_SIZE);
     let node = nodes.remove(0);
     let section_key_share = create_section_key_share(&sk2_set, 0);
+    let (used_space, root_storage_dir) = create_test_used_space_and_root_storage()?;
     let core = Core::new(
         create_comm().await?,
         node.clone(),
         section_full.clone(),
         Some(section_key_share),
         event_tx,
-    );
+        used_space,
+        root_storage_dir,
+    )?;
     let dispatcher = Dispatcher::new(core);
 
     let original_node_msg = NodeMsg::Sync {
@@ -1376,7 +1437,7 @@ async fn handle_bounced_untrusted_sync() -> Result<()> {
     Ok(())
 }
 
-#[tokio::test]
+#[tokio::test(flavor = "multi_thread")]
 async fn relocation_of_non_elder() -> Result<()> {
     relocation(RelocatedPeerRole::NonElder).await
 }
@@ -1401,13 +1462,16 @@ async fn relocation(relocated_peer_role: RelocatedPeerRole) -> Result<()> {
     let node_state = section_signed(sk_set.secret_key(), node_state)?;
     assert!(section.update_member(node_state));
     let node = nodes.remove(0);
+    let (used_space, root_storage_dir) = create_test_used_space_and_root_storage()?;
     let core = Core::new(
         create_comm().await?,
         node,
         section,
         Some(section_key_share),
         mpsc::channel(TEST_EVENT_CHANNEL_SIZE).0,
-    );
+        used_space,
+        root_storage_dir,
+    )?;
     let dispatcher = Dispatcher::new(core);
 
     let relocated_peer = match relocated_peer_role {
@@ -1474,12 +1538,12 @@ async fn relocation(relocated_peer_role: RelocatedPeerRole) -> Result<()> {
     Ok(())
 }
 
-#[tokio::test]
+#[tokio::test(flavor = "multi_thread")]
 async fn node_message_to_self() -> Result<()> {
     message_to_self(MessageDst::Node).await
 }
 
-#[tokio::test]
+#[tokio::test(flavor = "multi_thread")]
 async fn section_message_to_self() -> Result<()> {
     message_to_self(MessageDst::Section).await
 }
@@ -1501,7 +1565,9 @@ async fn message_to_self(dst: MessageDst) -> Result<()> {
         comm_tx,
     )
     .await?;
-    let core = Core::first_node(comm, node, event_tx)?;
+    let (used_space, root_storage_dir) = create_test_used_space_and_root_storage()?;
+
+    let core = Core::first_node(comm, node, event_tx, used_space, root_storage_dir)?;
     let node = core.node().clone();
     let section_pk = *core.section_chain().last_key();
     let dispatcher = Dispatcher::new(core);
@@ -1548,7 +1614,7 @@ async fn message_to_self(dst: MessageDst) -> Result<()> {
     Ok(())
 }
 
-#[tokio::test]
+#[tokio::test(flavor = "multi_thread")]
 async fn handle_elders_update() -> Result<()> {
     // Start with section that has `ELDER_SIZE` elders with age 6, 1 non-elder with age 5 and one
     // to-be-elder with age 7:
@@ -1602,13 +1668,16 @@ async fn handle_elders_update() -> Result<()> {
     };
 
     let (event_tx, mut event_rx) = mpsc::channel(TEST_EVENT_CHANNEL_SIZE);
+    let (used_space, root_storage_dir) = create_test_used_space_and_root_storage()?;
     let core = Core::new(
         create_comm().await?,
         node,
         section0.clone(),
         Some(section_key_share),
         event_tx,
-    );
+        used_space,
+        root_storage_dir,
+    )?;
     let dispatcher = Dispatcher::new(core);
 
     let commands = dispatcher
@@ -1670,7 +1739,7 @@ async fn handle_elders_update() -> Result<()> {
 }
 
 // Test that demoted node still sends `Sync` messages on split.
-#[tokio::test]
+#[tokio::test(flavor = "multi_thread")]
 async fn handle_demote_during_split() -> Result<()> {
     let node = create_node(MIN_ADULT_AGE, None);
     let node_name = node.name();
@@ -1707,13 +1776,16 @@ async fn handle_demote_during_split() -> Result<()> {
     }
 
     let (event_tx, _) = mpsc::channel(TEST_EVENT_CHANNEL_SIZE);
+    let (used_space, root_storage_dir) = create_test_used_space_and_root_storage()?;
     let core = Core::new(
         create_comm().await?,
         node,
         section,
         Some(section_key_share),
         event_tx,
-    );
+        used_space,
+        root_storage_dir,
+    )?;
     let dispatcher = Dispatcher::new(core);
 
     let sk_set_v1_p0 = SecretKeySet::random();
